<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns="http://earth.google.com/kml/2.1"
                xmlns:xhtml="http://www.w3.org/1999/xhtml"
                exclude-result-prefixes="xhtml"
                version="1.0">
 
    <xsl:output method="xml"/>

    <xsl:template match="/">
        <kml>
            <Document>
                <xsl:for-each select="//*[@class='vcard'][.//*[contains(@class, 'geo')]]">
                    <Placemark>
                        <name><xsl:apply-templates select=".//*[contains(@class, 'geo')]" mode="geo-name"/></name>
                        <description><xsl:text disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:apply-templates select="." mode="description"/><xsl:text disable-output-escaping="yes">]]&gt;</xsl:text></description>
                        <Point>
                            <coordinates><xsl:apply-templates select=".//*[contains(@class, 'geo')]" mode="geo-coordinates"/></coordinates>
                        </Point>
                    </Placemark>
                </xsl:for-each>
            </Document>
        </kml>
    </xsl:template>

    <xsl:template match="*" mode="geo-name">
        <xsl:value-of select="normalize-space(.)"/>
    </xsl:template>

    <xsl:template match="*" mode="geo-coordinates">
        <xsl:apply-templates select="self::xhtml:abbr"/>
    </xsl:template>

    <xsl:template match="xhtml:abbr" mode="geo-coordinates">
        <xsl:value-of select="concat(normalize-space(substring-after(@title,';')),',',normalize-space(substring-before(@title,';')),',0')"/>
    </xsl:template>

    <xsl:template match="*" mode="description">
        <xsl:if test=".//*[contains(@class,'adr')]">
            <xsl:apply-templates select=".//*[contains(@class,'adr')]" mode="adr"/>
        </xsl:if>
        <xsl:if test=".//*[contains(@class,'tel')]">
            <xsl:apply-templates select=".//*[contains(@class,'tel')]" mode="tel"/>
        </xsl:if>
    </xsl:template>

    <xsl:template match="*" mode="adr">
        <xsl:if test=".//*[@class='street-address']">
            <xsl:value-of select="normalize-space(.//*[@class='street-address'])"/><xsl:text disable-output-escaping="yes">&lt;BR></xsl:text>
        </xsl:if>
        <xsl:if test=".//*[@class='locality']">
            <xsl:value-of select="normalize-space(.//*[@class='locality'])"/><xsl:text>, </xsl:text>
        </xsl:if>
        <xsl:if test=".//*[@class='region']">
            <xsl:value-of select="normalize-space(.//*[@class='region'])"/><xsl:text> </xsl:text>
        </xsl:if>
        <xsl:if test=".//*[@class='postal-code']">
            <xsl:value-of select="normalize-space(.//*[@class='postal-code'])"/><xsl:text disable-output-escaping="yes">&lt;BR></xsl:text>
        </xsl:if>
    </xsl:template>

    <xsl:template match="*" mode="tel">
        <xsl:value-of select="normalize-space(.)"/><xsl:text disable-output-escaping="yes">&lt;BR></xsl:text>
    </xsl:template>

</xsl:stylesheet>